#include "xbasic_types.h"
#include "xio.h"

#include "xintc_l.h"
#include "xuartlite_l.h"

#define VGA_START_ADDRESS 0x00800000
#define SIZE              0x00000050
#define ADC_START         0x00c00000
#define DAC_START         0x00d00000
#define DELAY_START       0x00e00000


/* the main _interrupt_handler (writen by Xilinx):
1. solve the CPU system issues : saving / restoring context, etc. 
2. sees which interrupts are pending, and for each of them:
2a. acknowledges the interrupt
2b. calls a "user" device handler; their addresses are stores in a table

Our user device handlers are in audio_handler.c and uart_handler.c
I put them there not to keep code organized, but to be able to load them in the LBM
*/

// defined in uart_handler.c
extern volatile int uart_no_interrupts;
extern void uart_handler(void *callback);

// defined in audio_handler.c
//extern Xuint16 audio_adc_buf[], audio_dac_buf[];
extern volatile int audio_dac_bcnt_w, audio_dac_bcnt_r;
extern void audio_handler(void *callback);

extern int dac_rdptr,dac_wrptr,dac_length, adc_rdptr,adc_wrptr,adc_length;
int haveData;
Xuint16 data;

// INTERRUPT  SETUP

// using interrupts at this low level is not trivial
// somebody may find some logic in the following sequence

void setup_interrupts()
{

  // reset the interrupt controller

  // this disables the INTC irq output, so the INTC and CPU are "disconnected"
  XIntc_mMasterDisable(XPAR_INTC_SINGLE_BASEADDR);

  // disable all interrupt sources
  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR,0); // disable all

  // ack all sources, to be sure none of them is pending due to circuit wakeup
  XIntc_mAckIntr(XPAR_INTC_SINGLE_BASEADDR, 0xffffffff); // ack all sources


  // initialize the drivers and interrupt handlers
  // here we are also supposed to reset the uart etc. TO DO

  // AUDIO AK4565 DRIVER SETUP

//  XIntc_InterruptVectorTable[XPAR_INTC_AK4565_INTERRUPT_INTR].Handler=audio_handler;
  XIntc_RegisterHandler(XPAR_INTC_SINGLE_BASEADDR, XPAR_INTC_AK4565_INTERRUPT_INTR, audio_handler, NULL);


  audio_dac_bcnt_w=audio_dac_bcnt_r=0;

  // UART DRIVER & INTERRUPT SETUP


//  XIntc_InterruptVectorTable[XPAR_INTC_MYUART_INTERRUPT_INTR].Handler=uart_handler;
//XIntc_RegisterHandler(XPAR_INTC_SINGLE_BASEADDR, XPAR_INTC_MYUART_INTERRUPT_INTR, uart_handler, NULL);


  // DONT't forget to initialize your driver data structures !!
  uart_no_interrupts = 0;

  // enable the interrupts

  // enable the CPU interrupts
   microblaze_enable_interrupts();

  // enable the INTC irq output 
  XIntc_mMasterEnable(XPAR_INTC_SINGLE_BASEADDR); // master enable

  // enable the INTC to receive interrupts from the AK4565 and UART
  //  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR, 
  //		    XPAR_AK4565_INTERRUPT_MASK | XPAR_MYUART_INTERRUPT_MASK);
  
  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR, XPAR_AK4565_INTERRUPT_MASK);

  // enable the UART to generate interrupts
  // the audio driver always generate interrupts
  //XUartLite_mEnableIntr(XPAR_MYUART_BASEADDR);
 
}


// we prepare a square wave for audio output

/*
Xuint16 tri_wave[1024];
void make_triangle_wave()
{
  int i;
  for(i=0;i<512;i++) tri_wave[i]=tri_wave[1023-i]=(i-256)<<7;
}

// this functions ilustrates the user code
// it does 2 things:
// - plays a syrene sound
// - echoes all chars received from the uart

void make_some_noise()
{
  int phasel, phaser, dir, freq,i;
  char c;

  // clear_screen();

  make_triangle_wave();

  phasel=phaser=0; dir=0; freq=10000;
  while(1){

    if (audio_dac_bcnt_w - audio_dac_bcnt_r < 400)
      for(i=0;i<400;i++){
	audio_dac_buf[audio_dac_bcnt_w++ & 0x3FF] = tri_wave[(phaser>>11) & 0x3ff] +
	  (tri_wave[(phasel>>11) & 0x3FF] << 16);

	phasel+= 40000 - freq;
	phaser+=freq;

	if(dir){freq++; if(freq>30000) dir=0;} else
	  {freq--; if(freq<10000) dir=1;}
      }


    putnum(uart_no_interrupts); print("\n");
    
  }

}

*/

main()
{

  int i,a;
  adc_rdptr=0;
  adc_wrptr=0;
  haveData=0;
  adc_length=0;
  dac_rdptr=0;
  dac_wrptr=0;
  dac_length=0;

  print("Hello World!\r\n");

  for(a=0; a<640*480;a++)
    XIo_Out8(VGA_START_ADDRESS+a, 0);

  // Ok, the code is 6 times faster if cached
  //  microblaze_enable_icache();

  for(i=0;i<100;i++)
    XIo_Out8(VGA_START_ADDRESS+i+i*640, 0xff);

  setup_interrupts();

  for(i=0;i<100;i++)
    XIo_Out8(VGA_START_ADDRESS+100+i+i*640, 0xff);

  //  USER CODE 
  //  print("Hello interrupts\n\r");

  //   make_some_noise();

   for(i=0;i<100;i++)
    XIo_Out8(VGA_START_ADDRESS+200+i+i*640, 0xff);

   while(1)
   {
     //read from ADC FIFO
     //print("Reading from ADC FIFO\n\r");
     if(adc_length>0)
     {
       //data = audio_adc_buf[adc_rdptr];
       data = XIo_In16(ADC_START + (adc_rdptr<<1));

       adc_rdptr++;

       microblaze_disable_interrupts();
	 adc_length--;
	 microblaze_enable_interrupts();

       haveData=1;
       //print("\n\rThe data is ");
       //putnum(data);
       //print("\n\rThe adc_rdptr is ");
       //putnum(adc_rdptr);
       if(adc_rdptr==SIZE)
			 adc_rdptr=0;
     }
     else
       haveData=0;



     //write to DAC FIFO
     //print("Writing to DAC FIFO");
     if(haveData==1)
     {
       //audio_dac_buf[dac_wrptr] = data;
       XIo_Out16(DAC_START + (dac_wrptr<<1), data);

       dac_wrptr++;

		microblaze_disable_interrupts();
		dac_length++;
		microblaze_enable_interrupts();

       if(dac_wrptr==SIZE)
	 		dac_wrptr=0;
       if(dac_wrptr==dac_rdptr)
       {

	    	//for(i=0;i<100;i++)
	     	// XIo_Out8(VGA_START_ADDRESS+300+i+i*640, 0xff);
	 		//putnum(dac_wrptr);
	 		//print("DAC Buffer Overflow!!!");
	 		//putnum(dac_wrptr);
       }
     }
     

   }//end while
}//end main
     


