-------------------------------------------------------------------------------
-- xsb300_wrapper.vhd
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

library UNISIM;
use UNISIM.VCOMPONENTS.ALL;

library opb_xsb300_v1_00_a;
use opb_xsb300_v1_00_a.all;

entity xsb300_wrapper is
  port (
    OPB_Clk : in std_logic;
    OPB_Rst : in std_logic;
    OPB_ABus : in std_logic_vector(0 to 31);
    OPB_BE : in std_logic_vector(0 to 3);
    OPB_DBus : in std_logic_vector(0 to 31);
    OPB_RNW : in std_logic;
    OPB_select : in std_logic;
    OPB_seqAddr : in std_logic;
    UIO_DBus : out std_logic_vector(0 to 31);
    UIO_errAck : out std_logic;
    UIO_retry : out std_logic;
    UIO_toutSup : out std_logic;
    UIO_xferAck : out std_logic;
    PB_A : out std_logic_vector(19 downto 0);
    PB_LB_N : out std_logic;
    PB_UB_N : out std_logic;
    PB_D : inout std_logic_vector(15 downto 0);
    PB_WE_N : out std_logic;
    PB_OE_N : out std_logic;
    RAM_CE_N : out std_logic;
    ETHERNET_CS_N : out std_logic;
    ETHERNET_RDY : in std_logic;
    ETHERNET_IREQ : in std_logic;
    ETHERNET_IOCS16_N : in std_logic;
    pixel_clock : in std_logic;
    io_clock : in std_logic;
    VIDOUT_CLK : out std_logic;
    VIDOUT_RCR : out std_logic_vector(9 downto 0);
    VIDOUT_GY : out std_logic_vector(9 downto 0);
    VIDOUT_BCB : out std_logic_vector(9 downto 0);
    VIDOUT_BLANK_N : out std_logic;
    VIDOUT_HSYNC_N : out std_logic;
    VIDOUT_VSYNC_N : out std_logic
  );
  attribute x_core_info : STRING;
  attribute x_core_info of xsb300_wrapper: entity is "opb_xsb300_v1_00_a";

end xsb300_wrapper;

architecture STRUCTURE of xsb300_wrapper is

  component opb_xsb300 is
    generic (
      C_OPB_AWIDTH : integer;
      C_OPB_DWIDTH : integer;
      C_BASEADDR : std_logic_vector;
      C_HIGHADDR : std_logic_vector
    );
    port (
      OPB_Clk : in std_logic;
      OPB_Rst : in std_logic;
      OPB_ABus : in std_logic_vector(0 to C_OPB_AWIDTH-1);
      OPB_BE : in std_logic_vector(0 to C_OPB_DWIDTH/8-1);
      OPB_DBus : in std_logic_vector(0 to C_OPB_DWIDTH-1);
      OPB_RNW : in std_logic;
      OPB_select : in std_logic;
      OPB_seqAddr : in std_logic;
      UIO_DBus : out std_logic_vector(0 to C_OPB_DWIDTH-1);
      UIO_errAck : out std_logic;
      UIO_retry : out std_logic;
      UIO_toutSup : out std_logic;
      UIO_xferAck : out std_logic;
      PB_A : out std_logic_vector(19 downto 0);
      PB_LB_N : out std_logic;
      PB_UB_N : out std_logic;
      PB_D : inout std_logic_vector(15 downto 0);
      PB_WE_N : out std_logic;
      PB_OE_N : out std_logic;
      RAM_CE_N : out std_logic;
      ETHERNET_CS_N : out std_logic;
      ETHERNET_RDY : in std_logic;
      ETHERNET_IREQ : in std_logic;
      ETHERNET_IOCS16_N : in std_logic;
      pixel_clock : in std_logic;
      io_clock : in std_logic;
      VIDOUT_CLK : out std_logic;
      VIDOUT_RCR : out std_logic_vector(9 downto 0);
      VIDOUT_GY : out std_logic_vector(9 downto 0);
      VIDOUT_BCB : out std_logic_vector(9 downto 0);
      VIDOUT_BLANK_N : out std_logic;
      VIDOUT_HSYNC_N : out std_logic;
      VIDOUT_VSYNC_N : out std_logic
    );
  end component;

begin

  xsb300 : opb_xsb300
    generic map (
      C_OPB_AWIDTH => 32,
      C_OPB_DWIDTH => 32,
      C_BASEADDR => X"00800000",
      C_HIGHADDR => X"00FFFFFF"
    )
    port map (
      OPB_Clk => OPB_Clk,
      OPB_Rst => OPB_Rst,
      OPB_ABus => OPB_ABus,
      OPB_BE => OPB_BE,
      OPB_DBus => OPB_DBus,
      OPB_RNW => OPB_RNW,
      OPB_select => OPB_select,
      OPB_seqAddr => OPB_seqAddr,
      UIO_DBus => UIO_DBus,
      UIO_errAck => UIO_errAck,
      UIO_retry => UIO_retry,
      UIO_toutSup => UIO_toutSup,
      UIO_xferAck => UIO_xferAck,
      PB_A => PB_A,
      PB_LB_N => PB_LB_N,
      PB_UB_N => PB_UB_N,
      PB_D => PB_D,
      PB_WE_N => PB_WE_N,
      PB_OE_N => PB_OE_N,
      RAM_CE_N => RAM_CE_N,
      ETHERNET_CS_N => ETHERNET_CS_N,
      ETHERNET_RDY => ETHERNET_RDY,
      ETHERNET_IREQ => ETHERNET_IREQ,
      ETHERNET_IOCS16_N => ETHERNET_IOCS16_N,
      pixel_clock => pixel_clock,
      io_clock => io_clock,
      VIDOUT_CLK => VIDOUT_CLK,
      VIDOUT_RCR => VIDOUT_RCR,
      VIDOUT_GY => VIDOUT_GY,
      VIDOUT_BCB => VIDOUT_BCB,
      VIDOUT_BLANK_N => VIDOUT_BLANK_N,
      VIDOUT_HSYNC_N => VIDOUT_HSYNC_N,
      VIDOUT_VSYNC_N => VIDOUT_VSYNC_N
    );

end architecture STRUCTURE;

