LIBRARY unisim;
USE unisim.vcomponents.ALL;

--------------------------------------------------------------------------------
-- bram_elaborate.vhd
--------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

-- synthesis translate_off
LIBRARY UNISIM;
USE UNISIM.VCOMPONENTS.ALL;
-- synthesis translate_on

ENTITY bram_elaborate IS
GENERIC (
	C_MEMSIZE : INTEGER := 2048;
	C_PORT_DWIDTH : INTEGER := 32;
	C_PORT_AWIDTH : INTEGER := 32;
	C_NUM_WE : INTEGER := 4;
	C_FAMILY : STRING := "spartan2e");
PORT (
	-- instance GLOBAL
	BRAM_Dout_B : IN STD_LOGIC_VECTOR(0 TO 31);
	BRAM_Rst_A : IN STD_LOGIC;
	BRAM_Addr_B : IN STD_LOGIC_VECTOR(0 TO 31);
	BRAM_Dout_A : IN STD_LOGIC_VECTOR(0 TO 31);
	BRAM_WEN_A : IN STD_LOGIC_VECTOR(0 TO 3);
	BRAM_EN_B : IN STD_LOGIC;
	BRAM_WEN_B : IN STD_LOGIC_VECTOR(0 TO 3);
	BRAM_Clk_B : IN STD_LOGIC;
	BRAM_Clk_A : IN STD_LOGIC;
	BRAM_Din_A : OUT STD_LOGIC_VECTOR(0 TO 31);
	BRAM_Din_B : OUT STD_LOGIC_VECTOR(0 TO 31);
	BRAM_Rst_B : IN STD_LOGIC;
	BRAM_EN_A : IN STD_LOGIC;
	BRAM_Addr_A : IN STD_LOGIC_VECTOR(0 TO 31));
END bram_elaborate;

ARCHITECTURE IMP OF bram_elaborate IS

--------------------------------------------------------------------------------
COMPONENT RAMB4_S8_S8 IS
PORT (
	DIB : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
	WEB : IN STD_LOGIC;
	RSTB : IN STD_LOGIC;
	DOB : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
	ENB : IN STD_LOGIC;
	WEA : IN STD_LOGIC;
	ADDRB : IN STD_LOGIC_VECTOR(8 DOWNTO 0);
	DIA : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
	ADDRA : IN STD_LOGIC_VECTOR(8 DOWNTO 0);
	RSTA : IN STD_LOGIC;
	CLKA : IN STD_LOGIC;
	CLKB : IN STD_LOGIC;
	DOA : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
	ENA : IN STD_LOGIC);
END COMPONENT;

--------------------------------------------------------------------------------
-- internal signals
--------------------------------------------------------------------------------

	SIGNAL dina : STD_LOGIC_VECTOR(31 DOWNTO 0);

	SIGNAL dinb : STD_LOGIC_VECTOR(31 DOWNTO 0);

	SIGNAL douta : STD_LOGIC_VECTOR(31 DOWNTO 0);

	SIGNAL doutb : STD_LOGIC_VECTOR(31 DOWNTO 0);

	SIGNAL net_gnd0 : STD_LOGIC;

BEGIN

--------------------------------------------------------------------------------
-- Power assignments
net_gnd0 <= '0';
--------------------------------------------------------------------------------
-- Constant assignments
--------------------------------------------------------------------------------
-- Top-level port assignments
dinb <= BRAM_Dout_B;
dina <= BRAM_Dout_A;
BRAM_Din_A <= douta;
BRAM_Din_B <= doutb;
--------------------------------------------------------------------------------
-- Lower-level assignments

--------------------------------------------------------------------------------
ramb4_s8_s8_0 : RAMB4_S8_S8
PORT MAP (
	DIB => dinb(31 DOWNTO 24),
	WEB => BRAM_WEN_B(0),
	RSTB => net_gnd0,
	DOB => doutb(31 DOWNTO 24),
	ENB => BRAM_EN_B,
	WEA => BRAM_WEN_A(0),
	ADDRB => BRAM_Addr_B(21 TO 29),
	DIA => dina(31 DOWNTO 24),
	ADDRA => BRAM_Addr_A(21 TO 29),
	RSTA => net_gnd0,
	CLKA => BRAM_Clk_A,
	CLKB => BRAM_Clk_B,
	DOA => douta(31 DOWNTO 24),
	ENA => BRAM_EN_A);

--------------------------------------------------------------------------------
ramb4_s8_s8_1 : RAMB4_S8_S8
PORT MAP (
	DIB => dinb(23 DOWNTO 16),
	WEB => BRAM_WEN_B(1),
	RSTB => net_gnd0,
	DOB => doutb(23 DOWNTO 16),
	ENB => BRAM_EN_B,
	WEA => BRAM_WEN_A(1),
	ADDRB => BRAM_Addr_B(21 TO 29),
	DIA => dina(23 DOWNTO 16),
	ADDRA => BRAM_Addr_A(21 TO 29),
	RSTA => net_gnd0,
	CLKA => BRAM_Clk_A,
	CLKB => BRAM_Clk_B,
	DOA => douta(23 DOWNTO 16),
	ENA => BRAM_EN_A);

--------------------------------------------------------------------------------
ramb4_s8_s8_2 : RAMB4_S8_S8
PORT MAP (
	DIB => dinb(15 DOWNTO 8),
	WEB => BRAM_WEN_B(2),
	RSTB => net_gnd0,
	DOB => doutb(15 DOWNTO 8),
	ENB => BRAM_EN_B,
	WEA => BRAM_WEN_A(2),
	ADDRB => BRAM_Addr_B(21 TO 29),
	DIA => dina(15 DOWNTO 8),
	ADDRA => BRAM_Addr_A(21 TO 29),
	RSTA => net_gnd0,
	CLKA => BRAM_Clk_A,
	CLKB => BRAM_Clk_B,
	DOA => douta(15 DOWNTO 8),
	ENA => BRAM_EN_A);

--------------------------------------------------------------------------------
ramb4_s8_s8_3 : RAMB4_S8_S8
PORT MAP (
	DIB => dinb(7 DOWNTO 0),
	WEB => BRAM_WEN_B(3),
	RSTB => net_gnd0,
	DOB => doutb(7 DOWNTO 0),
	ENB => BRAM_EN_B,
	WEA => BRAM_WEN_A(3),
	ADDRB => BRAM_Addr_B(21 TO 29),
	DIA => dina(7 DOWNTO 0),
	ADDRA => BRAM_Addr_A(21 TO 29),
	RSTA => net_gnd0,
	CLKA => BRAM_Clk_A,
	CLKB => BRAM_Clk_B,
	DOA => douta(7 DOWNTO 0),
	ENA => BRAM_EN_A);

END ARCHITECTURE IMP;
