-------------------------------------------------------------------------------
-- CSEE 4840 Embedded System Design
-- Jaycam
-- Yaniv Schiller - yhs2001@columbia.edu
-- Avrum Tilman - amt77@columbia.edu
-- Josh Weinberg - jmw211@columbia.edu
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library UNISIM;
use UNISIM.VComponents.all;

entity svga is
    Port ( clk : in std_logic;
           pix_clk : in std_logic;
           rst : in std_logic;
           video_data : in std_logic_vector(15 downto 0);
           video_addr : out std_logic_vector(19 downto 0);
           video_req : out std_logic;
           VIDOUT_CLK : out std_logic;
           VIDOUT_RCR : out std_logic_vector(9 downto 0);
           VIDOUT_GY : out std_logic_vector(9 downto 0);
           VIDOUT_BCB : out std_logic_vector(9 downto 0);
           VIDOUT_BLANK_N : out std_logic;
           VIDOUT_HSYNC_N : out std_logic;
           VIDOUT_VSYNC_N : out std_logic);
end svga;

architecture Behavioral of svga is

component OBUF_F_12
port (O : out STD_ULOGIC;
		I : in STD_ULOGIC);
end component;

component FD
port (C : in std_logic;
      D : in std_logic;
      Q : out std_logic);
end component;

attribute iob : string;
attribute iob of FD : component is "true";

component svga_timing
port (h_synch_delay : out std_logic;
		v_synch_delay : out std_logic;
		comp_synch : out std_logic;
		blank : out std_logic;
		vga_ram_read_address : out std_logic_vector (19 downto 0);
		pixel_clock : in std_logic;
		reset : in std_logic);
end component;

signal r: std_logic_vector (9 downto 0);
signal g: std_logic_vector (9 downto 0);
signal b: std_logic_vector (9 downto 0);
signal blank, hsync, vsync : std_logic;
signal vga_ram_read_address : std_logic_vector(19 downto 0);
signal vreq, vreq_1, ldnshift : std_logic;
signal vga_shreg : std_logic_vector(15 downto 0);

begin

st : svga_timing
port map (	pixel_clock => pix_clk,
				reset => rst,
				h_synch_delay => hsync,
				v_synch_delay => vsync,
				blank => blank,
				vga_ram_read_address => vga_ram_read_address);

vreq <= not vga_ram_read_address(0);     -- also active on blanking, to fix
process(pix_clk)
begin
  if pix_clk'event and pix_clk='1' then
    vreq_1 <= vreq;
    ldnshift <= vreq_1;
  end if;
end process;  
  
video_req <= vreq;
video_addr <= '0' & vga_ram_read_address(19 downto 1);


process (pix_clk)
begin
  	if pix_clk'event and pix_clk='1' then
          if ldnshift = '1' then
            vga_shreg <= video_data;
          else
            vga_shreg <= vga_shreg(7 downto 0) & "00000000";
          end if;
        end if;
end process;

r(9 downto 7) <= vga_shreg (15 downto 13);
r(6 downto 0) <= "0000000";
g(9 downto 7) <= vga_shreg (12 downto 10);
g(6 downto 0) <= "0000000";
b(9 downto 8) <= vga_shreg (9 downto 8);
b(7 downto 0) <= "00000000";



-- from here nothing intelligent, just handling the I/O signals
-- i.e. the signals which are _physicaly_ on the chip pins
--     and travel on the pcb copper
-- 

vidclk : OBUF_F_12
port map (O => VIDOUT_clk,
	 I => pix_clk);

hsync_ff : FD port map (
  C => pix_clk,
  D => hsync, -- not
  Q => VIDOUT_HSYNC_N );

vsync_ff : FD port map (
  C => pix_clk,
  D => vsync, -- not
  Q => VIDOUT_VSYNC_N );

blank_ff :  FD port map (
  C => pix_clk,
  D =>  blank, -- not
  Q => VIDOUT_BLANK_N );

r_ff : for i in 0 to 9 generate
  r_ff : FD port map (
  C => pix_clk,
  D => r(i),
  Q => VIDOUT_RCR(i) );
end generate;

g_ff : for i in 0 to 9 generate
  g_ff : FD port map (
  C => pix_clk,
  D => g(i),
  Q => VIDOUT_GY(i) );
end generate;
  
b_ff : for i in 0 to 9 generate
  b_ff : FD port map (
  C => pix_clk,
  D => b(i),
  Q => VIDOUT_BCB(i) );
end generate;
  
end Behavioral;



